@@ -188,7 +188,7 @@ def install_packages_sequentially_async(
             '''
             results[pkg] = status == 'success'
             if on_output:
-                on_output(f'{pkg} {"✔️" if status == 'success' else "⚠️"}')
+                on_output(f'{pkg} {"✔️" if status == "success" else "⚠️"}')
             install_next(index + 1)
 
         current_proc = install_packages_async(
@@ -218,6 +218,7 @@ def install_packages_with_dialog(
     textview.set_wrap_mode(Gtk.WrapMode.WORD_CHAR)
     textbuffer = textview.get_buffer()
     scrolled = Gtk.ScrolledWindow()
+    scrolled.set_vexpand(True)
     scrolled.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
     scrolled.add(textview)
     vbox.pack_start(scrolled, True, True, 6)
@@ -225,17 +226,25 @@ def install_packages_with_dialog(
     progressbar.set_show_text(True)
     progressbar.set_text('0%')
     vbox.pack_start(progressbar, False, False, 6)
-    button_box = dialog.get_action_area()
+    action_area = Gtk.Box()
+    action_area.set_orientation(Gtk.Orientation.HORIZONTAL)
+    action_area.set_halign(Gtk.Align.END)
+    action_area.set_valign(Gtk.Align.CENTER)
+    action_area.set_can_focus(False)
+    action_area.set_hexpand(True)
+    action_area.set_vexpand(False)
+    action_area.set_spacing(0)
+    vbox.pack_start(action_area, False, False, 6)
     cancel_button = Gtk.Button()
     cancel_button_label = Gtk.Label()
     cancel_button_label.set_text_with_mnemonic(_('_Cancel'))
     cancel_button.add(cancel_button_label)
-    button_box.pack_start(cancel_button, False, False, 6)
+    action_area.pack_end(cancel_button, False, False, 6)
     close_button = Gtk.Button()
     close_button_label = Gtk.Label()
     close_button_label.set_text_with_mnemonic(_('_Close'))
     close_button.add(close_button_label)
-    button_box.pack_start(close_button, False, False, 6)
+    action_area.pack_end(close_button, False, False, 6)
     dialog.show_all()
     close_button.hide()
 
@@ -278,7 +287,9 @@ def install_packages_with_dialog(
             progressbar.set_fraction(total_percent / (package_count * 200))
             progressbar.set_text(f'{int(total_percent / (package_count * 2))}%')
             text = top_label.get_text()
-            top_label.set_text(f'{text + "\n" if text else ""}{line}')
+            # In Python < 3.12, f-string expression part cannot include a backslash
+            newline = '\n'
+            top_label.set_text(f'{text + newline if text else ""}{line}')
 
     def finish(status: InstallStatus) -> None:
         '''Installation of all packages finished or cancelled.'''
