@@ -176,7 +176,7 @@ class SimpleGtkTestCase(unittest.TestCase):
             self._gsettings.set_int('autoselectcandidate', 0)
             self._gsettings.set_int('candidatesdelaymilliseconds', 0)
         self.__class__.glib_main_loop = GLib.MainLoop()
-        Gtk.init()
+        Gtk.init() # pylint: disable=no-value-for-parameter
 
     def tearDown(self) -> None:
         if self.__class__.glib_main_loop is not None:
@@ -296,7 +296,10 @@ class SimpleGtkTestCase(unittest.TestCase):
         printflush('__engine_after_reset() called')
 
     def __entry_focus_in_event_cb(
-            self, entry: Gtk.Entry, event: Gdk.EventFocus) -> bool:
+            self,
+            entry: Gtk.Entry,
+            event: Gdk.EventFocus, # pylint: disable=c-extension-no-member
+    ) -> bool:
         if self.__test_index == len(TestCases['tests']):
             if DONE_EXIT and self.__class__.glib_main_loop is not None:
                 self.__class__.glib_main_loop.quit()
@@ -439,7 +442,7 @@ class SimpleGtkTestCase(unittest.TestCase):
         self.__main_test()
 
     def create_window(self) -> None:
-        window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
+        window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL) # pylint: disable=c-extension-no-member
         self.__entry = entry = Gtk.Entry()
         if self.__class__.glib_main_loop is not None:
             window.connect('destroy', self.__class__.glib_main_loop.quit)
